// Copyright (c) 2021 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "../libs.h"
#include "../engine/engine.h"
#include "../locale/language.h"

#if SETTING_GAME == SETTING_GAME_POKER
  #include "poker/pokergame.h"
#elif SETTING_GAME == SETTING_GAME_DAWN
  #include "dawn/dawngame.h"
#elif SETTING_GAME == SETTING_GAME_SANDBOX
  #include "sandbox/sandboxscene.h"
#endif

/** Describes the current game */
typedef struct {
  /** Engine for the game */
  engine_t engine;

  #if SETTING_GAME == SETTING_GAME_POKER
    pokergame_t pokerGame;
  #elif SETTING_GAME == SETTING_GAME_DAWN
    dawngame_t dawnGame;
  #elif SETTING_GAME == SETTING_GAME_SANDBOX
    sandboxscene_t sandboxScene;
  #endif
} game_t;

/**
 * Initialize the game context.
 * 
 * @return True if successful, otherwise false.
 */
bool gameInit(game_t *game);

/**
 * Tick the main game loop.
 * 
 * @param platformDelta The platform tick delta between the last render.
 * @return True if successful, false if safe exit requested..
 */
bool gameUpdate(game_t *game, float platformDelta);

/**
 * Cleanup the game instance.
 */
void gameDispose(game_t *game);