/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "pokerui.h"

void pokerUiInit(pokergame_t *pokerGame) {
  uint8_t i, j;

  // Initialize card render(s)
  imageInit(&pokerGame->ui.card);

  // Initialize players
  j = 0;
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    if(i == POKER_PLAYER_HUMAN_INDEX) continue;
    pokerPlayerUiInit(pokerGame->ui.player + j);
    j++;
  }
}

void pokerUiUpdate(pokergame_t *pokerGame, engine_t *engine) {
  uint8_t i, j;
  pokerplayerui_t *ui;

  j = 0;
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    if(i == POKER_PLAYER_HUMAN_INDEX) continue;
    ui = pokerGame->ui.player + j;
    pokerPlayerUiUpdate(ui, pokerGame, &pokerGame->assets.shader, i, engine);
    j++;
  }
}

void pokerUiRender(pokergame_t *pokerGame, engine_t *engine) {
  uint8_t i, j;
  pokerplayerui_t *ui;
  pokerplayer_t *player;

  char message[128];

  // cards
  if(pokerGame->poker.state >= POKER_STATE_DEALING) {
    for(j = 0; j < POKER_PLAYER_COUNT; j++) {
      player = pokerGame->poker.players + j;
      for(i = 0; i < player->cardCount; i++) {
        pokerCardSetImage(&pokerGame->ui.card, &pokerGame->assets.cardTexture, player->cards[i]);
        imageRender(&pokerGame->ui.card, &pokerGame->assets.shader, i * 64.0f, j * 100);
      }
    }
  }

  // show uh
  player = pokerGame->poker.players + POKER_PLAYER_HUMAN_INDEX;
  if(pokerGame->poker.bet.better == POKER_PLAYER_HUMAN_INDEX) {
    sprintf(message, "Press down to fold, up to bet, right to check/call.");
    labelSetText(&pokerGame->ui.player->label, &pokerGame->assets.font, message);
    labelRender(&pokerGame->ui.player->label, &pokerGame->assets.shader, 300, 100);
  }


  j = 0;
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    if(i == POKER_PLAYER_HUMAN_INDEX) continue;
    ui = pokerGame->ui.player + j;

    pokerPlayerUiRender(
      ui, pokerGame, &pokerGame->assets.shader, &pokerGame->assets.font, engine, 
      i, engine->render.width - ui->grid.width, j * 75.0f
    );
    j++;
  }
}

void pokerUiDispose(pokergame_t *pokerGame) {
  uint8_t i, j;

  imageDispose(&pokerGame->ui.card);

  j = 0;
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    if(i == POKER_PLAYER_HUMAN_INDEX) continue;
    pokerPlayerUiDispose(pokerGame->ui.player + j);
    j++;
  }
}